/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tudarmstadt.maki.simonstrator.api.component.sensor.environment.costs;

import java.util.ArrayList;
import java.util.List;

import de.tudarmstadt.maki.simonstrator.api.component.sensor.location.Location;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.roadnetwork.RoadNetwork;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.roadnetwork.RoadNetworkEdge;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.roadnetwork.RoadNetworkRoute;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.roadnetwork.paths.VehiclePathTracker;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.roadnetwork.paths.VehiclePathTrackerFactory;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.roadnetwork.routing.DijkstraAlgorithm;
import de.tudarmstadt.maki.simonstrator.api.component.vehicular.roadnetwork.routing.RoutingAlgorithm;

public class VehicularDetouringDecisionPoint extends VehicularDecisionPoint {

	private RoutingAlgorithm routing = new DijkstraAlgorithm();

	private static final boolean USE_STATIC_BENEFIT_FOR_DETOUR = true;

	private RoadNetworkEdge _detourLocation;

	public VehicularDetouringDecisionPoint(RoadNetworkEdge pDetourLocation, RoadNetworkEdge pEventLocation,
			Location pLocation, double pBenefit) {
		super(pEventLocation, pLocation, pBenefit);
		_detourLocation = pDetourLocation;
	}

	@Override
	public RoadNetworkEdge getDecisionPoint() {
		return _detourLocation;
	}

	public double calculateBenefit(RoadNetworkRoute pPath) {
		VehiclePathTracker tracker = VehiclePathTrackerFactory.getVehiclePathTracker();

		RoadNetworkEdge start = _detourLocation;
		RoadNetworkEdge destination = pPath.getDestination();
		RoadNetworkEdge edgeToAvoid = getEventEdge();

		List<RoadNetworkRoute> knownRoutes = new ArrayList<>();
		knownRoutes.add(pPath);
		List<RoadNetworkEdge> edgesToAvoid = new ArrayList<>();
		edgesToAvoid.add(edgeToAvoid);

		RoadNetworkRoute newRoute = routing.findRoute(RoadNetwork.CURRENT_ROAD_NETWORK, start, destination, knownRoutes,
				edgesToAvoid);
		if (newRoute == null) {
			return 0;
		}

		List<RoadNetworkEdge> oldSubEdges = new ArrayList<>();
		boolean add = false;
		for (RoadNetworkEdge roadNetworkEdge : pPath.getRoute()) {
			if (roadNetworkEdge.equals(start)) {
				add = true;
			}
			if (add) {
				oldSubEdges.add(roadNetworkEdge);
			}
		}
		RoadNetworkRoute oldRoute = new RoadNetworkRoute(oldSubEdges);

		double newRouteTravelTime = newRoute.getTravelTime();
		double routeTravelTimeWithEvent = oldRoute.getTravelTime();
		double routeTravelTimeWithoutEvent = oldRoute.getTravelTime();

		boolean found = false;
		for (int i = 0; i < oldRoute.getRoute().size() - 1; i++) {
			if (oldRoute.getRoute().get(i).equals(edgeToAvoid)) {
				routeTravelTimeWithEvent -= tracker.getTravelTime(oldRoute.getRoute().get(i),
						oldRoute.getRoute().get(i + 1));
				routeTravelTimeWithoutEvent -= tracker.getTravelTime(oldRoute.getRoute().get(i),
						oldRoute.getRoute().get(i + 1));
				found = true;
			}
		}
		if (found) {
			routeTravelTimeWithEvent += (edgeToAvoid.getLength() / edgeToAvoid.getMaxSpeed());
			routeTravelTimeWithoutEvent += (edgeToAvoid.getLength() / edgeToAvoid.getOriginalMaxSpeed());
		}

		if (USE_STATIC_BENEFIT_FOR_DETOUR) {
			return super.getBenefit() + (routeTravelTimeWithoutEvent - newRouteTravelTime) * 10;
		} else {
			return (routeTravelTimeWithEvent - newRouteTravelTime) * 10;
		}
	}

	public double calculateOverhead(RoadNetworkRoute pPath) {
		VehiclePathTracker tracker = VehiclePathTrackerFactory.getVehiclePathTracker();

		RoadNetworkEdge start = _detourLocation;
		RoadNetworkEdge destination = pPath.getDestination();
		RoadNetworkEdge edgeToAvoid = getEventEdge();

		List<RoadNetworkRoute> knownRoutes = new ArrayList<>();
		knownRoutes.add(pPath);
		List<RoadNetworkEdge> edgesToAvoid = new ArrayList<>();
		edgesToAvoid.add(edgeToAvoid);

		RoadNetworkRoute newRoute = routing.findRoute(RoadNetwork.CURRENT_ROAD_NETWORK, start, destination, knownRoutes,
				edgesToAvoid);
		if (newRoute == null) {
			return 0;
		}

		List<RoadNetworkEdge> oldSubEdges = new ArrayList<>();
		boolean add = false;
		for (RoadNetworkEdge roadNetworkEdge : pPath.getRoute()) {
			if (roadNetworkEdge.equals(start)) {
				add = true;
			}
			if (add) {
				oldSubEdges.add(roadNetworkEdge);
			}
		}
		RoadNetworkRoute oldRoute = new RoadNetworkRoute(oldSubEdges);

		double newRouteTravelTime = newRoute.getTravelTime();
		double routeTravelTimeWithoutEvent = oldRoute.getTravelTime();

		boolean found = false;
		for (int i = 0; i < oldRoute.getRoute().size() - 1; i++) {
			if (oldRoute.getRoute().get(i).equals(edgeToAvoid)) {
				routeTravelTimeWithoutEvent -= tracker.getTravelTime(oldRoute.getRoute().get(i),
						oldRoute.getRoute().get(i + 1));
				found = true;
			}
		}
		if (found) {
			routeTravelTimeWithoutEvent += (edgeToAvoid.getLength() / edgeToAvoid.getOriginalMaxSpeed());
		}

		return (routeTravelTimeWithoutEvent - newRouteTravelTime) * 10;
	}

	@Override
	public double getBenefit() {
		throw new UnsupportedOperationException();
	}

}
